﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

/* Roots of a polynomial by eigenvalues and QR decompostion                      */              
/*                                                                               */
/* x^6 - 0.02x^5 - 100.0003x^4 + 2.000004x^3 - 0.0300004x^2 - 0.0004x + 0.000004 */
/* www.mosismath.com                                                             */


namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {
        public const int MaxOrder = 7;      // order of teh polynomial
        public const int datapoints = 1000; // number of datapoints for the interpolation
 
        int count = 0;
      
        public double[,] y;
        public double[,] oldVal;
        public double[,] oldEigenVal;
        public int actOrder = MaxOrder-1;
       
        double[] yp = new double[datapoints];
        double[] xp = new double[datapoints];
        double[] y_k = new double[MaxOrder];  // suporting point in the graphics
        double[] pol = new double[MaxOrder];  // polynomial

        public QRalgorithm qr = new QRalgorithm(MaxOrder);
  
      
        public MainWin()
        {
            int i;
            DataGridViewCell cell;
            InitializeComponent();

            GPolynom.ColumnCount = MaxOrder;
            GPolynom.RowCount = 1;
            GEigenvals.ColumnCount = MaxOrder;
            GEigenvals.RowCount = 2;

            // Sample polynomial
            pol[0] = 1.0;   // x^6 - 0.02x^5 - 100.0003x^4 + 2.000004x^3 - 0.0300004x^2 - 0.0004x + 0.000004
            pol[1] = -0.02; // x + 0.01)^2 * (x - 0.02)^2 * (x^2 -100)
            pol[2] = -100.0003;
            pol[3] = 2.000004;
            pol[4] = 0.03000004;
            pol[5] = -0.0004;
            pol[6] = -0.000004;

            if (MaxOrder < 10)
            {
                for (i = 0; i < MaxOrder; i++)
                {
                    cell = GPolynom[i, 0];  // get cell to access
                    cell.Value = Convert.ToString(pol[i]);
                }
            }
        }

        public void Interpolate(int maxPoints, double minX, double maxX)
        {
          int i, k;
            for (i = 0; i < maxPoints; i++)
            {
                yp[i] = 0;
                xp[i] = ((double)(i) * (maxX - minX) / (double)maxPoints + minX);
                for (k = 0; k < MaxOrder; k++)
                {
                    yp[i] = yp[i] + pol[k] * Math.Pow(xp[i], (double)(MaxOrder - k - 1));
                }
            }
        }

        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            double[] x = new double[MaxOrder];
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double minTime = 1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (minTime > xp[j])
                        minTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = maxTime * 1.05;
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                if (maxVal > 0)
                    scalefactor = pGraph.Height / (maxVal) / 1.1;
                else
                    scalefactor = 1.0;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen blackPen = new Pen(Color.Black, 2);
            if (drawCeroline)
            {
                p1.X = 0;
                p1.Y = (pGraph.Height / 2);
                p2.X = pGraph.Width;
                p2.Y = (pGraph.Height / 2);
                g.DrawLine(blackPen, p1, p2);
            }
            bluePen.Width = 1;
            if (maxTime > 0) // draw interpolated graph
            {
                for (j = 0; j < maxPoints - 1; j++)
                {
                    if ((xp[j] >= minX) && (xp[j] <= maxX))
                    {
                        p1.X = Convert.ToInt32(((xp[j] - (double)(minTime)) / (double)(maxTime - minTime)) * pGraph.Width);
                        p2.X = Convert.ToInt32(((xp[j + 1] - (double)(minTime)) / (double)(maxTime - minTime)) * pGraph.Width);
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                            p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                        }
                        if (p2.X >= p1.X)
                            g.DrawLine(bluePen, p1, p2);
                    }
                }
                for (j = 0; j < MaxOrder; j++) // mark reference points
                {
                    float emSize = 10;
                    if (qr.valid[j])
                    {
                        p1.X = Convert.ToInt32((qr.cero[j].real - (double)(minTime)) * pGraph.Width / (double)(maxTime - minTime));
                        if (drawCeroline)
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                        else
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                        g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                        g.DrawString(Convert.ToString(Math.Round(qr.cero[j].real, 4)), new Font(FontFamily.GenericSansSerif, emSize, FontStyle.Regular),
                                      new SolidBrush(Color.Black), p1.X, p1.Y + 10);
                    }
                }
            }
            g.Dispose();
        }


        private void button1_Click(object sender, EventArgs e)
        {
            int i, k;
            DataGridViewCell cell;

            for (i = 0; i < actOrder; i++)
            {
                cell = GPolynom[i, 0];  // get cell to access
                pol[i] = Convert.ToDouble(cell.Value);
            }
            for (i = 0; i < actOrder; i++)
            {
                for (k = 0; k < actOrder; k++)
                {
                    qr.m.a[k, i].real = 0.0;
                    qr.m.a[k, i].imag = 0.0;
                }
                if (i < actOrder - 1)
                    qr.m.a[i + 1, i].real = 1.0;
            }
            for (i = 0; i < actOrder; i++)
                qr.m.a[i, actOrder - 1].real = -pol[MaxOrder - i - 1];
            actOrder = MaxOrder - 1;

            count = qr.run();  // qr decomposation

            for (i = 0; i < qr.cero.Length; i++)
            {
                qr.cero[i].real = qr.cero[i].real;
                cell = GEigenvals[i, 0];  // get cell to access
                if (qr.cero[i].real != 0)
                    cell.Value = Convert.ToString(Math.Round(qr.cero[i].real, 4));
                else
                    cell.Value = "";
                cell = GEigenvals[i, 1];  // get cell to access
                if (qr.cero[i].imag != 0)
                    cell.Value = Convert.ToString(Math.Round(qr.cero[i].imag, 4));
                else
                    cell.Value = "";

            }
            tbCount.Text = count.ToString();

            Interpolate(datapoints, qr.xk_min - 0.5, qr.xk_max + 0.5);
            DrawGraph(datapoints, qr.xk_min - 0.5, qr.xk_max + 0.5, true);
        }

        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, qr.xk_min - 0.5, qr.xk_max + 0.5, false);
        }  
    }
}
